/**************************************************************/
/*                                                            */
/* stm32_hw.c  -- Hardware dependent routines                 */ 
/*                                                            */
/*     STM32VL Discovery (STMicroelectronics, 24 MHz)         */
/*                                                            */
/* NON COMMERCIAL USE ONLY                                    */
/*                                                            */
/*   2011/5 Fujisekk LTD       (L)ym݌v                   */
/*      MGG03420@nifty.ne.jp                                  */
/*      http://www.d1.dion.ne.jp/~fujisekk                    */
/*      http://homepage3.nifty.com/fujisekk                   */
/*                                                            */
/**************************************************************/
#include "tonegen.h"
#include "stm32f10x.h"

#include <string.h>
#include <ST\iostm32f100xB.h>
#include <stm32f10x_dac.h>

#include "stm32f10x_tim.h"
#include "stm32f10x_rcc.h"
#include "misc.h"
//#include "STM32vldiscovery.h"

// #if (defined(STM32F103) | defined(STM32F10X_MD_VL))
uint32_t SystemCoreClock         = SYSCLK_FREQ_24MHz;        /*!< System Clock Frequency (Core Clock) */

volatile int wq_wr_ix, wq_rd_ix;

/**
  * @brief  Configures SysTick to generate an interrupt each 250ms.
  * @param  None
  * @retval None
  */
void SysTick_Configuration(void)
{
  /* SysTick interrupt each 250ms with counter clock equal to (HCLK/8) */
  if (SysTick_Config((SystemCoreClock/8) / 4))
  {
    /* Capture error */
    while (1);
  }

  /* Select HCLK/8 as SysTick clock source */
  SysTick_CLKSourceConfig(SysTick_CLKSource_HCLK_Div8);
}

/**
  * @brief  This function handles SysTick Handler.
  * @param  None
  * @retval None
  */
void SysTick_Handler(void)
{
  /* Reload IWDG counter */
  IWDG_ReloadCounter();

  /* Toggle LD4 */
 //??? STM32vldiscovery_LEDToggle(LED4);
  RCV_LED_TOGL
}


//void TIM6_DAC_IRQHandler(void)
void TIM6_DAC_IRQHandler(void)
//void MemManage_Handler(void)
{
  if (TIM_GetITStatus(TIM6, TIM_IT_Update) != RESET)
  {
    /* Set DAC Channel1 DHR register */
//    DAC_SetChannel1Data(DAC_Align_8b_R, wave_que[wq_rd_ix++]);
    DAC_SetChannel1Data(DAC_Align_12b_R, wave_que[wq_rd_ix++]);

    if (wq_rd_ix == WAVEQUESIZE)
    {
      wq_rd_ix = 0;
//      DFS_ReadFile(&fiwave, sector, Wavebuffer, &var, SECTOR_SIZE);
    }
 
   /* Clear TIM6 update interrupt */
    TIM_ClearITPendingBit(TIM6, TIM_IT_Update);
  }
}




void stm32_flash_setup( void )
{
// Embedded Flash Interface setup
#if !defined(STM32F10X_MD_VL)
  FLASH_ACR = 0x0012; // prefetch buffer enable, wait cycle = 2
#endif
//  FLASH_ACR = 0x0012; // prefetch buffer enable, wait cycle = 2
} // void stm32_flash_setup()

void stm32_nvic_setup( void )
{
// NVIC (Nested Vector Interrupt Controller)

//  SCB_ExceptionTableOffset =  0x00003000; // set vector table offset

//  SCB_VTOR =  0x00000000; // set vector table offset for STM32F100x
} // void stm32_nvic_setup()

void stm32_gpio_setup( void )
{
// GPIO setting

//    RCC_APB2ENR |= 0x11;     //CPIOCgpł悤ɂ
    RCC_APB2ENR |= 0x19;     //CCPIOCgpł悤ɂ
  GPIOA_BSRR = (1<<10) | (1<<8); //  pullup PA10, PA8

} // void stm32_gpio_setup()

void stm32_timer3_setup( void )
{
// Timer 3 setup ( fs generation )

  RCC_APB1ENR |= RCC_APB1Periph_TIM3; // enable clock for TIM3

  TIM3_CR1   = 0x94;  // down count, interrupt on OVF/UDF only
  TIM3_CR2   = TIM_CR2_MMS_1;  // master mode, TRGO at Update event
  TIM3_SMCR  = 0x00;  // slave mode disable
  TIM3_CCMR1 = 0x00;  // disable ch1, ch2 compare
  TIM3_CCMR2 = 0x6800; // ch3 = disable, ch4 = PWM1 mode, OC4 = out

  TIM3_PSC   = 0;       // set prescaler to 1/1
  TIM3_ARR   = 4500-1;  // set sampling frequency to 16 kHz
  TIM3_CCR4  = (4500 >> 1);    // duty register

  TIM3_CCER  = 0x3000; // OC4 enable and invert
  TIM3_DIER  = 0x0001; // enable UIE only
  NVIC->ISER[0] |= (1 << 29); // enable TIM3 interrupt   
//  NVIC_ISER[0] |= (1 << 29); // enable TIM3 interrupt   
  TIM3_CR1   |= 0x01;  // enable timer 3
  
// 
  SysTick_Config((SystemCoreClock/8) / 4);


} // void stm32_timer_setup()



  /* TIM6 Configuration */
  /* Time base configuration */
void stm32_timer6_setup( void )
{
  
 TIM_TimeBaseInitTypeDef TIM_TimeBaseStructure; //Initial data
 
  TIM_TimeBaseStructInit(&TIM_TimeBaseStructure);
  TIM_TimeBaseStructure.TIM_Period = 0x19;
  TIM_TimeBaseStructure.TIM_Prescaler = 0x0;
  TIM_TimeBaseStructure.TIM_ClockDivision = 0x0;
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up |0x80; //Auto Rerode
  TIM_TimeBaseInit(TIM6, &TIM_TimeBaseStructure);

  /* TIM6 TRGO selection */
  TIM_SelectOutputTrigger(TIM6, TIM_TRGOSource_Update);
  TIM_ITConfig(TIM6,1,ENABLE);/*???test Ŋ荞܂ꂽ*/
  NVIC_EnableIRQ(TIM6_DAC_IRQn); // enable TIM6 interrupt
}
/*Timer6 Sttart Stop */
void stm32_timer6_start( void )
{
  TIM6_CR1 |= 1;
}
void stm32_timer6_stop( void )
{
  TIM6_CR1 &= ~1;
}

void stm32_usart_setup( void )
{
// USART1 setup

  RCC_APB2ENR |= RCC_APB2Periph_USART1;  // enable clock for USART1
 
  //serial_midi = (0 != ((1<<8) & (GPIOA->IDR))); // test for PA8

    USART1_BRR  = 625;  // 24 MHz /  38.4 kbps = 625;
  USART1_CR1  = 0x2000;  // USRT1 enable, set 8-bit data, Non parity, no interrupt
  USART1_CR2  = 0x0000;  // set 1 stop bit
  USART1_CR3  = 0x0000;  // no RTS/CTS, no flow controll, no DMA
  USART1_CR1 |= 0x0C;    // RX, TX enable
} // void stm32_usart_setup()


void stm32_clock_setup( void )
{
// Clock configuration setup 

  RCC_CFGR = 0x0401C002     // PLLin=HSE, MCO=SYSCLK, ADC/8, APB2=HCLK, APB1=HCLK, AHB=SYSCLK, SYSCLK=PLL
            | RCC_PLLMul_3 ; // ( 8 MHz Xtal) x (PLLx3) = 24 MHz

  RCC_CR   = 0x01010001;  // PLL ON, HSE ON, HSI ON
  while (((1<<25) & (RCC_CR)) == 0);  // wait for PLL ready
} // void stm32_clock_setup()

/* TSEL defoult =Timer6*/
void stm32_dac_setup( void )
{
  RCC_APB1ENR |= RCC_APB1Periph_DAC; // enable clock for DAC
// DAC1 enable, trigger = TIM6_TRGO, Buffer off (for linearity)
  DAC_CR = (/*DAC_CR_TSEL1_0 | */DAC_CR_TEN1 | DAC_CR_EN1
#if !defined(USE_DACBUFOF) // defeat DAC buffer amp
		     | DAC_CR_BOFF1
#endif
		    );
} // void stm32_dac_setup()


void stm32_port_setup(void)
{
//  stm32_flash_setup();
//  stm32_clock_setup();
//  stm32_nvic_setup();
//  stm32_gpio_setup();
//  stm32_timer_setup();
// stm32_spi_setup();  
//  stm32_usart_setup();
  stm32_dac_setup();
} // void stm32_port_setup()



